#  -*- coding: iso-8859-1 -*-
# Copyright (C) 2007-2012  CEA/DEN, EDF R&D, OPEN CASCADE
#
# Copyright (C) 2003-2007  OPEN CASCADE, EADS/CCR, LIP6, CEA/DEN,
# CEDRAT, EDF R&D, LEG, PRINCIPIA R&D, BUREAU VERITAS
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
#

#  File   : Help.py
#  Author : Paul RASCLE, EDF
#  Module : SALOME
#  $Header: /home/server/cvs/KERNEL/KERNEL_SRC/src/KERNEL_PY/Help.py,v 1.5.2.2.14.2.12.1 2012-04-12 14:05:13 vsr Exp $
#
class SalomeDoc:
    def __init__(self, aDoc):
        self.doc = aDoc
    def __repr__(self):
        print self.doc
        return "---"
    def salome(self):
        doc_salome = '''
MODULE : salome
---------------
module salome gives access to Salome ressources:
variables:

  salome.orb             : CORBA
  salome.naming_service  : instance of naming Service class
      methods:
          Resolve(name)  : find a CORBA object (ior) by its pathname
          Register(name) : register a CORBA object under a pathname
  salome.lcc             : instance of lifeCycleCORBA class
      methods:
          FindOrLoadComponent(server,name) :
                           obtain an Engine (CORBA object)
                           or launch the Engine if not found,
                           with a Server name and an Engine name
  salome.sg
      methods:
         updateObjBrowser(bool):
         getActiveStudyId():
         getActiveStudyName():
         
         SelectedCount():      returns number of selected objects
         getSelected(i):       returns entry of selected object number i
         getAllSelected():     returns list of entry of selected objects
         AddIObject(Entry):    select an existing Interactive object
         RemoveIObject(Entry): remove object from selection
         ClearIObjects():      clear selection
         
         Display(*Entry):
         DisplayOnly(Entry):
         Erase(Entry):
         DisplayAll():
         EraseAll():

         IDToObject(Entry):    returns CORBA reference from entry

  salome.myStudyName     : active Study Name
  salome.myStudyId       : active Study Id
  salome.myStudy         : the active Study itself (CORBA ior)
                           methods : defined in SALOMEDS.idl
                                                         
methods:
  salome.DumpStudy(study) : Dump a study, given the ior
---
'''
        print doc_salome
        
    def geompy(self):
        doc_geompy = '''
MODULE : geompy
---------------
module geompy provides an encapsulation of GEOM Engine methods
variables:
  geompy.geom               : a Geometry Engine, found or loaded
                              at first import of module geompy.
                              methods : defined in GEOM_Gen.idl
  geompy.myBuilder          : a study builder
  geompy.father             : GEOM root in current study (salome.myStudy)

methods:
  addToStudy(aShape, aName) : add the shape into the current study
  --- all methods of GEOM_Gen.idl that returns a shape are encapsulated,
      with the same interface : shapes are named with their ior
'''
        print doc_geompy
        
    def supervision(self):
        doc_supervision = '''
MODULES : SALOME_SuperVisionEditor and SALOME_SuperVisionExecutor
-----------------------------------------------------------------
this modules provide access to Editor and Executor Engine methods

See SUPERV.idl

In order to run the example (supervisionexample.py)

    Type : from supervisionexample import *
           supervisionexample.py contains comments

A new python example avoids references to LifeCycleCORBA
                     avoids references to NamingService
                     avoids references to ModuleCatalog
                     avoids SuperVisionComponent creation
                     allows G.Input(...) instead of AddInput(G,...)
                     replaces Editor/Executor with Graph
                     allows Nodes, Ports and Links CORBA objects
                     shortens methods names
                     ...

    See /SuperVisionTest/resources/GraphExample.py
                                   and GraphExample.xml
---
'''
        print doc_supervision
        
    

Help = SalomeDoc('''
Availables modules:
  salome      : gives access to Salome ressources
  geompy      : encapsulation of GEOM Engine methods
  supervision : gives access to SuperVision Engine
To obtain specific help on a module "truc", type: Help.truc()
To run an example, type: import example3
''')
  



#  -*- coding: iso-8859-1 -*-
# Copyright (C) 2007-2012  CEA/DEN, EDF R&D, OPEN CASCADE
#
# Copyright (C) 2003-2007  OPEN CASCADE, EADS/CCR, LIP6, CEA/DEN,
# CEDRAT, EDF R&D, LEG, PRINCIPIA R&D, BUREAU VERITAS
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
#

#  File   : IDLparser.py
#  Module : SALOME

import string, sys, fpformat, re, os
import xml.sax
import pdb

from xml.sax.handler import *
from omniidl import idlast, idltype, idlvisitor, idlutil, output

# parameters not found in IDL file, user's specified in optional parameters
common_data={"AUTHOR"     : "",
             "ICON"       : "",
             "VERSION"    : "",
             "COMP_TYPE"  : "",
             "COMP_NAME"  : "",
             "COMP_UNAME" : "",
             "COMP_MULT"  : "",
             "COMP_IMPL"  : ""
             }

nb_components = 0

#--------------------------------------------------
# extract value of <param_name> from <args> list
# it's proposed that the matching <args> item
# looks like <param_name>=<value>, for example,
# catalog=/tmp/myxml.xml
#--------------------------------------------------
def getParamValue( param_name, default_value, args ):
    pattern="^"+param_name+"="

    res = default_value        #initial value
    for opt in args:
        s = re.compile(pattern).search(opt)
        if s:
            res = opt[s.end():]
            break     #found

    return res


#--------------------------------------------------
# print error message
#--------------------------------------------------
def error (message):
    print "ERROR : ", message


#--------------------------------------------------
# base class implementing tree
#--------------------------------------------------
class Tree:

    def __init__(self, name = '', content = '', key = None):
        self.name = name
        self.content = content
        self.key = key
        self.parent = None
        self.childs = []
        self.comments = []
        self.attrs={}

    def addChild(self, tree):
        if tree is not None:
            self.childs.append(tree)
            tree.parent = self
        return tree

    def addNamedChild(self, name, content = ''):
        return self.addChild(Tree(name, content))

    def replaceChild(self, tree):
         if tree is not None:
            pos = 0
            for i in self.childs:
                if (i.name == tree.name) & ((i.key is None) | (i.key == tree.key)):
                    self.childs.pop(pos)
                    self.childs.insert(pos, tree)
                    return tree
                pos += 1

         return self.addChild(tree)

    def insertFirstChild(self, tree):
        if tree is not None:
            self.childs.insert(0, tree)
        return tree

    def insertFirstNamedChild(self, name, content = ''):
        return self.insertFirstChild(Tree(name, content))

    def output_xml(self, f, depth=0):
        d = depth
        if self.name != '':
            s = string.ljust('', 4*depth)
            s += '<' + self.name
            for k,v in self.attrs.items():
              s += ' ' + k + '="' + v + '"'
            s += '>'
            if self.content != '':
                s +=  self.content
            else:
                if len(self.childs) > 0:
                    s += '\n'
            f.write(s)
            d +=  1

        for i in self.childs:
            i.output_xml(f, d)

        if self.name != '':
            s = '</' + self.name + '>\n'
            if len(self.childs) > 0 :
                s = string.ljust('', 4*depth) + s
            f.write(s)

    def Dump(self, levels=-1, depth=0):
        #Dumps the tree contents

        if levels == 0: return

        s = string.ljust('', 4*depth)
        print s, self, self.content
        for i in self.childs:
            i.Dump(levels-1, depth+1)

    def parents(self):
        #Returns list of the parents
        l = []
        p = self.parent
        while p:
            l.append(p)
            l.append(p.name)
            p = p.parent
        return l

    def getChild(self, name, content=None):
        # return child node with a given name
        # if content == None, don't compare contents
        for i in self.childs:
            if (i.name == name):
                if (content is None) | (i.content == content):
                    return i
        return None

    def getNode(self, name, content='', depth=-1):
        # recursive search of a node with a given name
        # content == None, don't compare content
        if (self.name == name):
            if (content is None) | (self.content == content):
                return self

        if (depth != 0):
            for i in self.childs:
                n = i.getNode(name, content, depth-1)
                if n:  return n

        return None

    def __repr__(self):
        s = '<'
        if self.name != '':
            s += self.name
        else:
            s +=  'None'
        s += '>'
        return s

    def merge(self, t):
        pass

    def mergeChilds(self, t, list):
        L_ext = t.getChild(list)
        L_int = self.getChild(list)

        L_merge = Tree(list)

        for i_ext in L_ext.childs:
            k_ext = i_ext.key
            if k_ext is None:  continue
            present = 0

            for i_int in L_int.childs:
                k_int = i_int.key
                if k_int is None:  continue

                if (k_int == k_ext):
                    present = 1
                    break;

            if present :
                i_int.merge(i_ext)
                L_merge.addChild(i_int)
            else:
                L_merge.addChild(i_ext)

        self.replaceChild(L_merge)

    def setAttrib(self, name,value):
      self.attrs[name]=value


#--------------------------------------------------
# implements parameter tree
#--------------------------------------------------
class parameter(Tree):

    def __init__(self, name=None, mode = 'in', type='', comment='unknown'):
        Tree.__init__(self, mode + 'Parameter', key=name)
        self.mode = mode
        if name is None:  return

        self.addNamedChild(mode + 'Parameter-name', name)
        self.addNamedChild(mode + 'Parameter-type', type)
        self.addNamedChild(mode + 'Parameter-comment', comment)

    def merge(self, P):

        self.mode = P.mode
        self.replaceChild(P.getChild(P.mode + 'Parameter-name'))
        self.replaceChild(P.getChild(P.mode + 'Parameter-type'))
        C = P.getChild(P.mode + 'Parameter-comment')
        if C.content != 'unkonwn':
            self.replaceChild(C)

#--------------------------------------------------
# implements dataStreamParameter tree
#--------------------------------------------------
class dataStreamParameter(parameter):

    def __init__(self, name=None, mode='in', type='', dependency='', comment='unknown'):
        parameter.__init__(self, name, mode, type, comment)
        if name is None:  return

        self.addNamedChild(mode + 'Parameter-dependency', dependency)
        self.mode = mode

    def merge(self, P):

        parameter.merge(self, P)
        self.replaceChild(P.getChild(mode + 'Parameter-dependency'))


def parseComment(comment):

    spaces = '[\t\n ]*'
    word = spaces + '([a-zA-Z][a-zA-Z0-9_]*)' + spaces

    result = []
    type = None
    key = None

    ## match :  // followed by a 'DataStreamPorts' string,
    ## the service name, and a list of ports
    pattern = '// *DataStreamPorts{,1}' + word
    m = re.match(pattern, comment)

    ## if there is a match, parse remaining part of comment
    if m:
        ## service
        type = 'DataStreamPorts'
        key = m.group(1)

        sPorts = comment[m.end():]
        pattern = word + '\('+word+','+word +','+word+'\)' \
                  + spaces + ',{,1}' + spaces
        while len(sPorts) > 0:
            ## process next DataStreamPort
            ## match a definition like xx(a,b,c) with a possible trailing ,
            ## returns a tuple (xx, a, b, c) and
            ## the remaining part of input string
            m = re.match(pattern, sPorts)
            if m is None:
                raise LookupError, \
                      'format error in DataStreamPort definition : '+sPorts
            sPorts = sPorts[m.end():]
            result.append(m.groups())

    return type, key, result;

#--------------------------------------------------
# implements service tree
#--------------------------------------------------
class Service(Tree):

    def __init__(self, name=None, comment = 'unknown'):

        Tree.__init__(self, 'component-service', key=name)
        if name is None:  return

        self.addNamedChild('service-name', name)
        self.addNamedChild('service-author',common_data["AUTHOR"])
        self.addNamedChild('service-version',common_data["VERSION"])
        self.addNamedChild('service-comment', comment)
        self.addNamedChild('service-by-default', "0")
        self.addNamedChild('inParameter-list')
        self.addNamedChild('outParameter-list')
        self.addNamedChild('DataStream-list')

    def createInParameter(self, name, type):
        L = self.getChild('inParameter-list')
        p = parameter(name, 'in', type)
        L.replaceChild(p)
        return p

    def createOutParameter(self, name, type):
        L = self.getChild('outParameter-list')
        p = parameter(name, 'out', type)
        L.replaceChild(p)
        return p

    def createDataStreamParameter(self, p):
        L = self.getChild('DataStream-list')
        p = dataStreamParameter(p[0], p[2], p[1], p[3])
        L.replaceChild(p)
        return p

    def merge(self, S):

        self.replaceChild(S.getChild('service-author'))
        self.replaceChild(S.getChild('service-version'))
        self.replaceChild(S.getChild('service-by-default'))
        C = S.getChild('service-comment')
        if C.content != 'unkonwn':
            self.replaceChild(C)

        for L in ['inParameter-list', 'outParameter-list', 'DataStream-list']:
           self.mergeChilds(S, L)


#--------------------------------------------------
# implements interface tree
#--------------------------------------------------
class Interface(Tree):

    def __init__(self, name=None, comment='unknown'):

        Tree.__init__(self, key=name)

        if name is None:  return

        self.addNamedChild('component-interface-name', name)
        self.addNamedChild('component-interface-comment', comment);
        self.addNamedChild('component-service-list')

    def createService(self, name):
        L = self.getChild('component-service-list')

        if L is None:
            error ("Interface.createService() : 'component-service-list' is not found")
            return None

        s = Service(name)
        L.addChild(s)
        return s

    def findService(self, key):
        L = self.getChild('component-service-list')
        for S in L.childs:
            if S.key == key:
                return S
        return None

    def merge(self, I):

        C = S.getChild('component-interface-comment')
        if C.content != 'unkonwn':
            self.replaceChild(C)

        self.mergeChilds(I, 'component-service-list')

    def processDataStreams(self):
        for sComment in self.comments:

            type, key, result = parseComment(sComment)

            if type == 'DataStreamPorts':
                Service = self.findService(key)
                if Service is None:
                    raise LookupError, \
                          'service ' + key + \
                          ' not found in interface : ' + self.key
                for p in result:
                ## process next DataStreamPort
                    Service.createDataStreamParameter(p)


#--------------------------------------------------
# implements Component tree
#--------------------------------------------------
class Component(Tree):
    def __init__(self, name=None):
        Tree.__init__(self, 'component', key=name)
        if name is None:  return

# ASV : fix for bug PAL8922 (Component name indicated by user in GUI is not taken into account
        if common_data["COMP_NAME"] != '':
            self.addNamedChild('component-name', common_data["COMP_NAME"])
        else:
            self.addNamedChild('component-name', name)

# ASV : if user name is NOT set, then use component-name instead.  Else - default.
        if common_data["COMP_UNAME"] != '':
            self.addNamedChild('component-username',   common_data["COMP_UNAME"])
        else:
            if common_data["COMP_NAME"] != '':
                self.addNamedChild('component-username', common_data["COMP_NAME"] )
            else:
                self.addNamedChild('component-username',   name)

        self.addNamedChild('component-type',       common_data["COMP_TYPE"])
        self.addNamedChild('component-author',     common_data["AUTHOR"])
        self.addNamedChild('component-version',    common_data["VERSION"])
        self.addNamedChild('component-comment',    'unknown')
        self.addNamedChild('component-multistudy', common_data["COMP_MULT"])
        self.addNamedChild('component-impltype',   common_data["COMP_IMPL"])
        self.addNamedChild('component-icone',      common_data["ICON"])
        self.addNamedChild('constraint')
        self.addNamedChild('component-interface-list')

    def createInterface(self, name):
        L = self.getChild('component-interface-list')
        if L is None:
            error("createInterface: No component-interface-list is found")
            return None
        i = Interface(name)
        L.addChild(i)
        return i

    def merge(self, C):

        for i in ['component-username', 'component-author',
                  'component-type', 'component-icone', 'component-version',
                  'component-multistudy', 'component-impltype', 'constraint']:
            ext = C.getChild(i)
            int = self.getChild(i)
            if int is None:
                int = ext
            elif ext is not None and len(ext.content):
                int.content = ext.content

        Cc = C.getChild('component-comment')
        if Cc.content != 'unkonwn':
            self.replaceChild(Cc)

        self.mergeChilds(C, 'component-interface-list')

#--------------------------------------------------
# implements document tree
#--------------------------------------------------
class Catalog(ContentHandler, Tree):
    def __init__(self, filename = None):
        Tree.__init__(self)
        self.buffer = ''
        self.list = []
        if (filename):
            parser = xml.sax.make_parser()
            parser.setContentHandler(self)
            parser.parse(filename)
        else:
            t = self.addNamedChild('begin-catalog')
            t.addNamedChild('type-list')
            t.addNamedChild('component-list')

        n = self.getChild('begin-catalog')
        if n is None:
            error("Catalog.__init__ : No 'begin-catalog' is found!")
            return
        if n.getChild('path-prefix-list') is None:
            n.insertFirstNamedChild('path-prefix-list')
        if n.getChild('type-list') is None:
            p=n.childs.index(n.getChild('path-prefix-list'))
            n.childs.insert(p+1,Tree('type-list'))
        if n.getChild('component-list') is None:
            n.addNamedChild('component-list')

    def removeComponent(self, name):
        complist = self.getNode('component-list')
        idx = 0
        if complist is None:
            print "Catalog.removeComponent() : 'component-list' is not found"
            return
        for comp in complist.childs:
            cname = comp.getChild('component-name')
            if cname is not None:
                if cname.content == name:
                    complist.childs.pop(idx)
                    print "Component " + name + " is removed"
            idx += 1

    def startDocument(self):
        self.list.append(self)

    def startElement(self, name, attrs):
        p = self.list[len(self.list)-1]
        if name == 'component':
            e = p.addChild(Component())
        elif name == 'component-interface-name':
            e = p.addNamedChild(name)
        elif name == 'component-service':
            e = p.addChild(Service())
        elif name == 'inParameter':
            e = p.addChild(parameter(mode='in'))
        elif name == 'outParameter':
            e = p.addChild(parameter(mode='out'))
        elif name == 'sequence':
            e = p.addChild(SeqType(attrs["name"],attrs["content"]))
        elif name == 'objref':
            e = p.addChild(ObjType(attrs["name"]))
        elif name == 'struct':
            e = p.addChild(StructType(attrs["name"]))
        elif name == 'type':
            e = p.addChild(Type(attrs["name"],attrs["kind"]))
        elif name == 'member':
            e = p.addChild(Member(attrs["name"],attrs["type"]))
        else:
            e = p.addNamedChild(name)
        self.list.append(e)
        self.buffer = ''

    def endElement(self, name):
        self.buffer = string.join(string.split(self.buffer), ' ')
        p = self.list[len(self.list)-1]
        p.content = self.buffer
        if name == 'component':
            p.key = p.getChild('component-name').content
        self.buffer = ''
        e = self.list.pop()

    def characters(self, ch):
        self.buffer += ch

    def mergeComponent(self, comp):

        L_int = self.getNode('component-list')
        if   L_int is None:
            error("Catalog.mergeComponent : 'component-list' is not found")
            return

        i_ext = comp
        present = 0
        n_ext = i_ext.key
        for i_int in L_int.childs:
            if (i_int.key == n_ext):
                present = 1
                break;

        if present == 0:
            print '   add component', i_ext.getChild('component-name').content
            L_int.addChild(i_ext)
        else:
            print '   replace component', i_ext.getChild('component-name').content
            i_int.merge(i_ext)

    def mergeType(self, type):
      L_int = self.getNode('type-list')
      if L_int is None:
        error("Catalog.mergeType : 'type-list' is not found")
        return
      for t in L_int.childs:
        if t.attrs["name"] == type.attrs["name"]:
          t.merge(type)
          return

      L_int.addChild(type)

class Member(Tree):
  def __init__(self, name,type):
    Tree.__init__(self, 'member')
    self.setAttrib("name",name)
    self.setAttrib("type",type)

class Type(Tree):
  def __init__(self, name,kind):
    Tree.__init__(self, 'type')
    self.setAttrib("name",name)
    self.setAttrib("kind",kind)

  def merge(self,t):
    self.setAttrib("kind",t.attrs["kind"])

class SeqType(Tree):
  def __init__(self, name,content):
    Tree.__init__(self, 'sequence')
    self.setAttrib("name",name)
    self.setAttrib("content",content)

  def merge(self,t):
    self.setAttrib("content",t.attrs["content"])

class StructType(Tree):
  def __init__(self, name):
    Tree.__init__(self, 'struct')
    self.setAttrib("name",name)

  def merge(self,t):
    #remove childs and replace by t childs
    self.childs=[]
    for c in t.childs:
      self.childs.append(c)

class ObjType(Tree):
  def __init__(self, name):
    Tree.__init__(self, 'objref')
    self.setAttrib("name",name)

  def merge(self,t):
    RepoId=t.attrs.get("id")
    if RepoId:
      self.setAttrib("id",RepoId)
    #remove childs and replace by t childs
    self.childs=[]
    for c in t.childs:
      self.childs.append(c)

# IDL file reader

ttsMap = {
    idltype.tk_void:       "void",
    idltype.tk_short:      "short",
    idltype.tk_long:       "long",
    idltype.tk_ushort:     "unsigned short",
    idltype.tk_ulong:      "unsigned long",
    idltype.tk_float:      "float",
    idltype.tk_double:     "double",
    idltype.tk_boolean:    "boolean",
    idltype.tk_char:       "char",
    idltype.tk_octet:      "octet",
    idltype.tk_any:        "any",
    idltype.tk_TypeCode:   "CORBA::TypeCode",
    idltype.tk_Principal:  "CORBA::Principal",
    idltype.tk_longlong:   "long long",
    idltype.tk_ulonglong:  "unsigned long long",
    idltype.tk_longdouble: "long double",
    idltype.tk_wchar:      "wchar"
    }


#--------------------------------------------------
# class ModuleCatalogVisitor
#--------------------------------------------------
class ModuleCatalogVisitor (idlvisitor.AstVisitor):

    def __init__(self, catalog):
        self.catalog = catalog
        self.EngineType = 0
        self.currentScope=None

    def visitAST(self, node):
        for n in node.declarations():
            if n.mainFile():
              n.accept(self)

    def visitModule(self, node):
        self.currentScope=node
        for n in node.definitions():
            n.accept(self)

    def visitInterface(self, node):
        if node.mainFile():

            self.EngineType = 0

            for i in node.inherits():
                s = i.scopedName();
                if s[0] == "Engines":
                  if s[1] == "EngineComponent":
                    self.EngineType = 1; break
                  if s[1] == "Superv_Component":
                    self.EngineType = 2; break

            if self.EngineType:
              #This interface is a SALOME component
              Comp = Component(node.identifier())

              self.currentInterface = Comp.createInterface(node.identifier())

              for c in node.callables():
                if isinstance(c, idlast.Operation):
                    c.accept(self)

              for c in node.declarations():
                if isinstance(c, idlast.Struct):
                    c.accept(self)

              for i in node.comments():
                self.currentInterface.comments.append(str(i))

              if self.EngineType == 2:
                self.currentInterface.processDataStreams()

              global nb_components
              nb_components = nb_components + 1
              self.catalog.mergeComponent(Comp)

            else:
              #This interface is not a component : use it as a DataType
              t=ObjType("/".join(node.scopedName()))
              for i in node.inherits():
                t.addNamedChild("base","/".join(i.scopedName()))
              self.catalog.mergeType(t)

            self.EngineType = 0


    def visitOperation(self, node):

        self.currentService = self.currentInterface.createService \
                                       (node.identifier())

        node.returnType().accept(self)
        if (self.currentType != "void"):
            self.currentService.createOutParameter \
                ("return", self.currentType)

        for c in node.parameters():
            c.accept(self)

        for i in node.comments():
            self.currentInterface.comments.append(str(i))


    def visitDeclaredType(self, type):
        name=type.name()
        scoped_name="/".join(type.scopedName())
        self.currentType = scoped_name

    def visitBaseType(self, type):
        self.currentType = ttsMap[type.kind()]

    def visitStringType(self, type):
        self.currentType = "string"

    def visitParameter(self, node):
        node.paramType().accept(self)
        if node.is_in():
            self.currentService.createInParameter \
                     (node.identifier(), self.currentType)
        if node.is_out():
            self.currentService.createOutParameter \
                     (node.identifier(), self.currentType)

    def visitSequenceType(self,type):
      type.seqType().accept(self)
      if type.bound() == 0:
          self.contentType=self.currentType
          self.currentType = "sequence"
      else:
          self.currentType = None

    def visitTypedef(self, node):
      if node.constrType():
            node.aliasType().decl().accept(self)

      node.aliasType().accept(self)
      type  = self.currentType
      if not type:
        return
      decll = []
      for d in node.declarators():
            d.accept(self)
            if self.__result_declarator:
              decll.append(self.__result_declarator)
      if type == "sequence":
        #it's a sequence type
        for name in decll:
          scoped_name="/".join(self.currentScope.scopedName()+[name])
          self.catalog.mergeType(SeqType(scoped_name,self.contentType))
      #else:
        #it's an alias
      #  for name in decll:
      #    scoped_name="/".join(self.currentScope.scopedName()+[name])
      #    self.catalog.mergeType(Type(scoped_name,type))

    def visitStruct(self, node):
      t=StructType("/".join(node.scopedName()))
      for m in node.members():
            if m.constrType():
                m.memberType().decl().accept(self)

            m.memberType().accept(self)
            type = self.currentType
            for d in m.declarators():
                d.accept(self)
                t.addChild(Member(self.__result_declarator,type))

      self.catalog.mergeType(t)

    def visitDeclarator(self, node):
        if node.sizes():
          self.__result_declarator =None
        else:
          self.__result_declarator =node.identifier()

#--------------------------------------------------
# parse idl and store xml file
#--------------------------------------------------
def run(tree, args):

    CatalogFileName=getParamValue("catalog", "CatalogModulePersonnel.xml", args)
    if re.compile(".*?.xml$").match(CatalogFileName, 1) is None:
        CatalogFileName = CatalogFileName + '.xml'

    #=========  Read parameters  ======================
    common_data["ICON"]       = getParamValue("icon",       "",                args)
    common_data["AUTHOR"]     = getParamValue("author",     os.getenv("USER"), args)
    common_data["VERSION"]    = getParamValue("version",    "1",               args)
    common_data["COMP_NAME"]  = getParamValue("name",       "",                args)
    common_data["COMP_UNAME"] = getParamValue("username",   "",                args)
    common_data["COMP_TYPE"]  = getParamValue("type",       "OTHER",           args)
    common_data["COMP_MULT"]  = getParamValue("multistudy", "1",               args)
    common_data["COMP_IMPL"]  = getParamValue("impltype",   "1",               args)

    print common_data

    remove_comp = getParamValue("remove", "", args)

    #==================================================

    if (os.path.exists(CatalogFileName)):
        print "Importing", CatalogFileName
        C = Catalog(CatalogFileName)
    else:
        print "Creating ",CatalogFileName
        C = Catalog()

    print "Reading idl file"

    visitor = ModuleCatalogVisitor(C)
    tree.accept(visitor)

##    C.Dump()

    if remove_comp :
        C.removeComponent(remove_comp)

    if (os.path.exists(CatalogFileName)):
        print "Updating", CatalogFileName
        CatalogFileName_old = CatalogFileName + '_old'
        os.rename(CatalogFileName, CatalogFileName_old)
    else:
        CatalogFileName_old = ""
        print "Writing", CatalogFileName

    CatalogFileName_new = CatalogFileName + '_new'
    f=open(CatalogFileName_new, 'w')
    f.write("<?xml version='1.0' encoding='us-ascii' ?>\n\n")
    C.output_xml(f)
    f.close()

    os.rename(CatalogFileName_new, CatalogFileName)
    if ((CatalogFileName_old != "") & os.path.exists(CatalogFileName_old)):
        os.unlink(CatalogFileName_old)

    print


if __name__ == "__main__":
    print
    print "Usage : omniidl -bIDLparser [-I<catalog files directory>]* -Wbcatalog=<my_catalog.xml>[,icon=<pngfile>][,version=<num>][,author=<name>][,name=<component_name>][,username=<component_username>][,multistudy=<component_multistudy>][,impltype=<implementation type : 0 (python), 1 (C++)>] <file.idl>"
    print



# -*- coding: iso-8859-1 -*-
# Copyright (C) 2007-2012  CEA/DEN, EDF R&D, OPEN CASCADE
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
#

__all__ = [ "deprecation", "logger", "termcolor", "logconfig" ]

# WARN: This file SHOULD NOT import salome stuff so that modules of the
# package could be used outside of a SALOME session context when
# possible. For example logger.py, datamodeler.py, enumerate.py,
# diclookup.py, ... does not need a SALOME running application.

#
# ==============================================================================
# Generic serialization functions for the module datamodeler
# ==============================================================================
#
import pickle

def serialize(dataToSerialize):
    """
    Creates a string description of the specified data.
    .. attribute:: dataToSerialize
    a data object to serialize
    """
    serialstring = pickle.dumps(dataToSerialize)
    return serialstring

def unserialize(stringToUnserialize):
    """
    Creates a data object from its string description. The string description
    is supposed to be obtained by a call to serialize.
    .. attribute:: stringToUnserialize
    a string description of the object
    """
    data = pickle.loads(stringToUnserialize)
    return data

class Callable:
    """
    This class is used to create class-method (see MetaData)
    """
    def __init__(self, anycallable):
        self.__call__ = anycallable
#
# ==============================================================================
# Basic use cases and unit tests
# ==============================================================================
#

def TEST_serialization():
    from testdata import TestData
    ref_name = "my study case"
    studyData = TestData()
    studyData.setName(ref_name)

    print "serialize data ..."
    serialString = serialize(studyData)
    print "unserialize data ..."
    unserialData = unserialize(serialString)

    res_name = unserialData.getName()
    print res_name
    if ( ref_name != res_name ):
        return False

    unserialData.log()
    unserialData.setName("an other name")

    return True

if __name__ == "__main__":
    import unittester
    unittester.run("salome/kernel/__init__","TEST_serialization")



# -*- coding: iso-8859-1 -*-
# Copyright (C) 2007-2012  CEA/DEN, EDF R&D, OPEN CASCADE
#
# Copyright (C) 2003-2007  OPEN CASCADE, EADS/CCR, LIP6, CEA/DEN,
# CEDRAT, EDF R&D, LEG, PRINCIPIA R&D, BUREAU VERITAS
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
#

__author__="gboulant"
__date__ ="$1 avr. 2010 18:12:38$"

import time
import threading

# ===========================================================================
class Runner(threading.Thread):
    """
    This class provides a tool to run and drive a function in a dedicated thread.
    """
    def __init__(self, functionToRun=None,*argv):
        threading.Thread.__init__( self )
        self._stopevent = threading.Event()
        self.setFunction(functionToRun)
        self.setArguments(*argv)
        self._exception = None
        self._return    = None
        self._observer  = None
        self._callbackFunction = None
        # We create an id from the name and the time date in milliseconds
        self._id = functionToRun.__name__ +"/"+ str(time.time())

    def setFunction(self,functionToRun):
        """
        Positionne la fonction � ex�cuter. La fonction peut �tre la
        m�thode d'un objet pass�e sous la forme 'monobjet.mamethode'.
        """
        self.clear()
        self._function = functionToRun

    def setArguments(self,*argv):
        """
        Positionne les arguments � passer � la fonction
        """
        self.clear()
        self._argv = argv

    def getReturn(self):
        """
        Retourne le resultat de la fonction. En cas d'erreur, on
        r�cup�rera l'exception lev�e au moyen de la m�thode
        getException().
        """
        return self._return

    def setCallbackFunction(self, callbackFunction):
        self._callbackFunction = callbackFunction

    def setObserver(self, observerToNotify):
        """
        Permet de sp�cifier un observateur � notifier en fin
        d'ex�cution. L'observateur est suppos� �tre un objet qui
        impl�mente une m�thode processNotification()
        """
        try:
            observerToNotify.processNotification
        except AttributeError:
            raise DevelException("The observer should implement the method processNotification()")

        self._observer = observerToNotify

    def run(self):
        """
        Ex�cution de la fonction. Impl�mentation de la m�thode run
        d�clench�e par l'appel � Thread.start().
        """
        print "##################### threadhelper.run"
        if self._function is None: return
        try:
            self._return = self._function(*self._argv)
        except Exception, e:
            print e
            self._exception = e
        self._stopevent.set()
        self.notifyObserver()
        self.callback()
        # _GBO_ Maybe it's no use to have both observers and a callback function.
        # One of the two mechanism should be sufficient

    def notifyObserver(self):
        if self._observer is None:
            # Aucune notification pr�vue
            return
        try:
            self._observer.processNotification()
        except AttributeError, att:
            if str(att) == "processNotification":
                print "L'observateur n'impl�mente pas la m�thode processNotification()"
            else:
                print "La fonction processNotification() a lev� une exception:"
                print att
        except Exception, e:
            print "La fonction processNotification() a lev� une exception:"
            print e

    def callback(self):
        if self._callbackFunction is None: return
        self._callbackFunction()

    def isEnded(self):
        """
        Retourne true si la fonction s'est termin�e naturellement
        (correctement ou en erreur). Utile pour les client qui
        pratique le pooling (interrogation r�p�t�e � intervalle
        r�gulier.
        """
        return self._stopevent.isSet()
        # _GBO_ On n'utilise pas isAlive() pour pouvoir ma�triser
        # l'indicateur de stop (exemple de la fonction kill)
        # return not self.isAlive()

    def getException(self):
        return self._exception

    def getId(self):
        return self._id

    def wait(self,timeout=None):
        """
        Met fin au thread apr�s timeout seconde s'il est encore en
        ex�cution.
        Si le compte-�-rebours est atteind sans que la fonction
        functionToRun soit termin�e, alors le runner est laiss� dans
        l'�tat 'not Ended', c'est-�-dire que isEnded retourne
        false. On peut ainsi distinguer l'arr�t normal de l'arr�t �
        l'issue du timeout.
        """
        threading.Thread.join(self, timeout)


    def kill(self,message="Arr�t demand�"):
        """
        Cette m�thode doit �tre appeler pour interrombre le
        thread. Cet appel d�clare le thread comme en erreur (exception
        mise � disposition par la m�thode getException().
        """
        # On cr�e un exception indiquant la demande d'interruption
        self._exception = Exception(message)
        self._stopevent.set()
        # _GBO_ ATTENTION
        # Un thread python ne peut pas en r�alit� �tre int�rrompu.
        # La fonction reste donc en ex�cution m�me si on a rejoint le
        # thread appelant.

    def clear(self):
        self._stopevent.clear()
        self._exception = None
        self._return    = None

# ===========================================================================
CONTINUE=1
STOP=0

class PeriodicTimer( threading.Thread ):
    """
    Cette classe permet d'amorcer un compte-�-rebours p�riodique pour
    op�rer un m�canisme de pooling. On d�finit la fonction appell�e
    p�riodiquement et la fonction terminale.
    """
    def __init__( self,  loopdelay, initdelay=0,
                  periodic_action=None,
                  ended_action=None,
                  *ended_argv ):

        if periodic_action is None:
            self._periodic_action = self.defaultPeriodicAction
        else:
            self._periodic_action = periodic_action
        self._ended_action = ended_action
        self._loopdelay = loopdelay
        self._initdelay = initdelay
        self._running = CONTINUE
        self._ended_argv    = ended_argv
        threading.Thread.__init__( self )

    def defaultPeriodicAction():
        """
        Les fonctions 'periodicAction' retournent CONTINU ou STOP
        apr�s avoir ex�cut� l'action de fin de boucle. Si STOP est
        retourn�, le cycle est interrompu. 
        """
        return CONTINU

    def run( self ):
        if self._initdelay:
            time.sleep( self._initdelay )
        self._runtime = time.time()
        while self._running == CONTINUE:
            start = time.time()
            self._running  = self._periodic_action()
            self._runtime += self._loopdelay
            time.sleep( max( 0, self._runtime - start ) )
        if self._ended_action is not None:
            self._ended_action( *self._ended_argv )

    def stop( self ):
        self._running = STOP



#
# ======================================================
# Fonctions de test unitaire
# ======================================================
#

# ======================================================
import os
testfilename="/tmp/threadhelperTestFile"
def testIfContinue():
    print "On examine la pr�sence du fichier ", testfilename
    if os.path.exists(testfilename):
        return STOP
    else:
        return CONTINUE

def endedAction():
    print "FINI"

def TEST_PeriodicTimer():
    periodicTimer=PeriodicTimer(1,0,testIfContinue, endedAction)
    periodicTimer.start()


# ======================================================
def function_ok(nbsteps=5):
    """
    Fonction qui se termine correctement
    """
    print "D�but"
    cnt=0
    while ( cnt < nbsteps ):
        print "Etape ", cnt
        time.sleep(0.6)
        cnt+=1

    print "Fin"

def function_with_exception():
    """
    Fonction qui aboutie � une lev�e d'exception
    """
    print "D�but"
    cnt=0
    while ( cnt < 5 ):
        print "Etape ", cnt
        time.sleep(1)
        cnt+=1
    
    raise Exception("erreur d'ex�cution de la fonction")
    print "Fin"

def infinite_function():
    """
    fonction de dur�e infinie (tant qu'il y a du courant �l�ctrique) pour
    le test du timeout.
    """
    print "D�but"
    cnt=0
    while ( 1 ):
        print "Etape ", cnt
        time.sleep(1)
        cnt+=1
    
    raise Exception("erreur")
    print "Fin"


def runWithRunner(functionToRun):
    """
    Ex�cute la fonction avec le runner. On illustre ici la modalit�
    d'utilisation du Runner.
    """
    print "###########"
    runner = Runner(functionToRun)
    runner.start()

    while ( not runner.isEnded() ):
        print "La fonction est en cours"
        time.sleep(0.2)
    e = runner.getException()
    if e is not None:
        print "La fonction s'est termin�e en erreur"
        print e
        # On peut en fait la relancer
        # raise e
    else:
        print "La fonction s'est termin�e correctement"


def runWithTimeout(functionToRun, timeout=10):
    """
    Ex�cute la fonction avec le runner. On illustre ici la modalit�
    d'utilisation du Runner.
    """
    print "runWithTimeout : DEBUT"
    runner = Runner(functionToRun)
    runner.start()

    # On se fixe un temps au del� duquel on consid�re que la fonction
    # est en erreur => on tue le thread (timeout)
    runner.wait(timeout)
    print "Apr�s runner.timeout(timeout)"
    if not runner.isEnded():    
        runner.kill()
    e = runner.getException()
    if e is not None:
        print "La fonction s'est termin�e en erreur"
        print e
        # On peut en fait la relancer
        # raise e
    else:
        print "La fonction s'est termin�e correctement"

    print "runWithTimeout : FIN"
    import sys
    sys.exit(0)
    

def TEST_Timeout():
    #runWithTimeout(function_ok)
    #runWithTimeout(function_ok,2)
    runWithTimeout(function_with_exception)

    
def TEST_Runner():
    runWithRunner(function_ok)
    runWithRunner(function_with_exception)
    #runWithRunner(infinite_function)


def myCallbackFunction():
    print "myCallbackFunction: the job is ended"
    

def TEST_runWithCallback():
    runner = Runner(function_ok,8)
    runner.setCallbackFunction(myCallbackFunction)
    runner.start()

    if runner.getException() is not None:
        return False

    runnerId = runner.getId()
    print "A runner has been started with id="+str(runnerId)
    cpt = 0
    while ( not runner.isEnded() ):
        print "Waiting notification from process "+str(runner.getId())+", step n°"+str(cpt)
        time.sleep(0.2)
        cpt+=1

    return True

if __name__ == "__main__":
    import unittester
    #TEST_PeriodicTimer()
    #TEST_Runner()
    #TEST_Timeout()
    unittester.run("threadhelper","TEST_runWithCallback")

    



#  -*- coding: iso-8859-1 -*-
# Copyright (C) 2007-2012  CEA/DEN, EDF R&D, OPEN CASCADE
#
# Copyright (C) 2003-2007  OPEN CASCADE, EADS/CCR, LIP6, CEA/DEN,
# CEDRAT, EDF R&D, LEG, PRINCIPIA R&D, BUREAU VERITAS
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
#

#  File   : salome_study.py
#  Author : Paul RASCLE, EDF
#  Module : SALOME
#  $Header: /home/server/cvs/KERNEL/KERNEL_SRC/src/KERNEL_PY/salome_study.py,v 1.6.2.2.14.4.12.1 2012-04-12 14:05:13 vsr Exp $
#
import salome_kernel
import SALOMEDS
import salome_iapp
from launchConfigureParser import verbose

#--------------------------------------------------------------------------

def DumpComponent(Study, SO, Builder,offset):
  it = Study.NewChildIterator(SO)
  while it.More():
    CSO = it.Value()
    a=offset*"--" + ">" + CSO.GetID()
    find,AtName = Builder.FindAttribute(CSO, "AttributeName")
    if find:
      a=a+":"+AtName.Value()
    find,AtIOR = Builder.FindAttribute(CSO, "AttributeIOR")
    if find:
      a=a+":"+AtIOR.Value()
    find,RefSO = CSO.ReferencedObject()
    if find:
      a=a+":"+RefSO.GetID()
    print a
    DumpComponent(Study, CSO, Builder,offset+2)
    it.Next()

#--------------------------------------------------------------------------

def DumpStudy(Study):
    """
    Dump a study, given the ior
    """
    itcomp = Study.NewComponentIterator()
    Builder = Study.NewBuilder()
    while itcomp.More():
      SC = itcomp.Value()
      name = SC.ComponentDataType()
      print "-> ComponentDataType is " + name
      DumpComponent(Study, SC,Builder, 1)
      itcomp.Next()

def DumpStudies():
  """
    Dump all studies in a StudyManager
  """
  for name in myStudyManager.GetOpenStudies():
    s=myStudyManager.GetStudyByName(name)
    print "study:",name, s._get_StudyId()
    DumpStudy(s)


#--------------------------------------------------------------------------

def IDToObject(id):
    myObj = None
    mySO = myStudy.FindObjectID(id);
    if mySO is not None:
        ok, anAttr = mySO.FindAttribute("AttributeIOR")
        if ok:
            AtIOR = anAttr._narrow(SALOMEDS.AttributeIOR)
            if AtIOR.Value() != "":
                myObj = orb.string_to_object(AtIOR.Value())
    return myObj

def ObjectToSObject(obj):
    mySO = None
    if obj is not None:
        ior =  orb.object_to_string(obj)
        if ior != "":
            mySO = myStudy.FindObjectIOR(ior)
    return mySO

def ObjectToID(obj):
    mySO = ObjectToSObject(obj)
    if mySO:
        return mySO.GetID()
    return ""

def IDToSObject(id):
    mySO = myStudy.FindObjectID(id);
    return mySO

    #--------------------------------------------------------------------------

def generateName(prefix = None):
    import random;
    int = random.randint(1,1000);
    if prefix is None:
        return "Study" + str(int)
    else :
        return prefix + str(int)

    #--------------------------------------------------------------------------

def PersistentPresentation(theStudy, theSO, theWithID):
    # put the sobject's content (with subchildren) to the string
    aResult = ""
    attrs = theSO.GetAllAttributes()
    aLen = len(attrs)
    anUncopied = 0
    for a in range(0,aLen):
        attr = attrs[a]
        if isinstance(attr,SALOMEDS._objref_AttributeTreeNode):
            anUncopied += 1
        elif isinstance(attr,SALOMEDS._objref_AttributeTarget):
            anUncopied += 1
        elif isinstance(attr,SALOMEDS._objref_AttributeReal) or \
             isinstance(attr,SALOMEDS._objref_AttributeInteger) or \
             isinstance(attr,SALOMEDS._objref_AttributeName) or \
             isinstance(attr,SALOMEDS._objref_AttributeComment) or \
             isinstance(attr,SALOMEDS._objref_AttributePersistentRef) or \
             isinstance(attr,SALOMEDS._objref_AttributeLocalID) or \
             isinstance(attr,SALOMEDS._objref_AttributeUserID):
            aResult += " attribute value: " + str(attr.Value())
        elif isinstance(attr,SALOMEDS._objref_AttributeIOR):
            aResult += " attribute: IOR"
        elif isinstance(attr,SALOMEDS._objref_AttributeSequenceOfReal) or \
             isinstance(attr,SALOMEDS._objref_AttributeSequenceOfInteger):
            aResult += " Sequence: " + str(attr.CorbaSequence())
        elif isinstance(attr,SALOMEDS._objref_AttributeDrawable):
            aResult += " Drawable: " + str(attr.IsDrawable())
        elif isinstance(attr,SALOMEDS._objref_AttributeSelectable):
            aResult += " Selectable: " + str(attr.IsSelectable())
        elif isinstance(attr,SALOMEDS._objref_AttributeExpandable):
            aResult += " Expandable: " + str(attr.IsExpandable())
        elif isinstance(attr,SALOMEDS._objref_AttributeOpened):
            aResult += " Opened: " + str(attr.IsOpened())
        elif isinstance(attr,SALOMEDS._objref_AttributeTextColor):
            aResult += " TextColor: " + str(attr.TextColor())
        elif isinstance(attr,SALOMEDS._objref_AttributeTextHighlightColor):
            aResult += " TextHighlightColor: " + str(attr.TextHighlightColor())
        elif isinstance(attr,SALOMEDS._objref_AttributePixMap):
            aResult += " PixMap: " + str(attr.GetPixMap())
        elif isinstance(attr,SALOMEDS._objref_AttributeTableOfInteger) or \
             isinstance(attr,SALOMEDS._objref_AttributeTableOfReal):
            aResult += " Table with title: " + attr.GetTitle()
        elif isinstance(attr,SALOMEDS._objref_AttributePythonObject):
            aResult += " PythonObject: " + attr.GetObject()

    if theWithID:
        aResult = "sobject: " + theSO.GetID() + " nbattrs: " + str(aLen - anUncopied) + aResult + '\n'
    else:
        aResult = " nbattrs: " + str(aLen - anUncopied) + aResult + '\n'
    anIter = theStudy.NewChildIterator(theSO)
    while anIter.More():
        aResult += PersistentPresentation(theStudy, anIter.Value(), theWithID)
        anIter.Next()
    return aResult

    #--------------------------------------------------------------------------

def GetTree(theSO):
    # returns the document list tree (as list)
    aResult = [theSO.GetID()]
    anIter = myStudy.NewChildIterator(theSO)
    while anIter.More():
        aResult += GetTree(anIter.Value())
        anIter.Next()
    return aResult

    #--------------------------------------------------------------------------

def CheckCopyPaste(theSO, theInfo ,theComponentPaste):
    aRoot = theSO
    while aRoot.GetID() != "0:":
        aRoot = aRoot.GetFather()
    aTree = GetTree(aRoot)
    aStudyPersist = PersistentPresentation(myStudy, aRoot, 1)

    if not myStudyManager.CanCopy(theSO):
        raise RuntimeError, "<CanCopy> for "+theInfo+" returns false"
    
    if not myStudyManager.Copy(theSO):
        raise RuntimeError, "<Copy> for "+theInfo+" returns false"

    
    if not myStudyManager.CanPaste(theSO):
        raise RuntimeError, "<CanPaste> for "+theInfo+" returns false"

    # check: before paste study is not changed check
    if aStudyPersist != PersistentPresentation(myStudy, aRoot, 1):
        raise RuntimeError, "Study is changed before Paste calling for "+theInfo
    
    aSObj = theSO
    if theComponentPaste:
        aSObj = theSO.GetFatherComponent()
        theInfo = theInfo + "(paste for component)"
    if myStudyManager.Paste(aSObj) == None:
        raise RuntimeError, "<Paste> for "+theInfo+" returns None object"
    aNewTree = GetTree(aRoot)
    aLen = len(aTree)
    for a in range(0,aLen):
        if aTree[a] != aNewTree[a]:
            return myStudy.FindObjectID(aNewTree[a])
        
    if aLen < len(aNewTree):
        return myStudy.FindObjectID(aNewTree[aLen])
    
    raise RuntimeError, "After Copy calling the tree is not changed"
    
    #--------------------------------------------------------------------------

def FindFileInDataDir(filename):
    import os
    datadir = os.getenv("DATA_DIR")
    if datadir is not None:
        import string
        dirs = string.split(datadir, ":")
        for dir in dirs:
            file = dir + "/" + filename
            if os.path.exists(file):
                return file;
    datadir = os.getenv("KERNEL_ROOT_DIR") + "/examples/"
    file = datadir + filename
    if os.path.exists(file):
        return file;

    return None

    #--------------------------------------------------------------------------

salome_study_ID = -1

def getActiveStudy(theStudyId=0):
    global salome_study_ID

    if verbose(): print "getActiveStudy"
    if salome_study_ID == -1:
        if salome_iapp.hasDesktop():
            if verbose(): print "---in gui"
            salome_study_ID = salome_iapp.sg.getActiveStudyId()
        else:
            if verbose(): print "---outside gui"
            if theStudyId:
                aStudy=myStudyManager.GetStudyByID(theStudyId)
                if aStudy:
                    if verbose(): print "connection to existing study ", theStudyId
                    salome_study_ID = theStudyId
            if salome_study_ID == -1:
                salome_study_ID = createNewStudy()
            if verbose(): print"--- Study Id ", salome_study_ID
    return salome_study_ID

    #--------------------------------------------------------------------------

def setCurrentStudy(theStudy):
    """
    Change current study : an existing one given by a study object.

    :param theStudy: the study CORBA object to set as current study
    """
    global myStudyId, myStudy, myStudyName
    global salome_study_ID
    myStudy=theStudy
    myStudyId=theStudy._get_StudyId()
    myStudyName=theStudy._get_Name()
    return myStudyId, myStudy, myStudyName

    #--------------------------------------------------------------------------

def setCurrentStudyId(theStudyId=0):
    """
    Change current study : an existing or new one.
    optional argument : theStudyId
        0      : create a new study (default).
        n (>0) : try connection to study with Id = n, or create a new one
                 if study not found.
    """
    global myStudyId, myStudy, myStudyName
    global salome_study_ID
    salome_study_ID = -1
    myStudyId = getActiveStudy(theStudyId)
    if verbose(): print "myStudyId",myStudyId
    myStudy = myStudyManager.GetStudyByID(myStudyId)
    myStudyName = myStudy._get_Name()

    return myStudyId, myStudy, myStudyName

    #--------------------------------------------------------------------------

def createNewStudy():
    print "createNewStudy"
    i=1
    aStudyName = "noName"
    nameAlreadyInUse = 1
    listOfOpenStudies = myStudyManager.GetOpenStudies()
    print listOfOpenStudies
    while nameAlreadyInUse:
        aStudyName = "extStudy_%d"%i
        if aStudyName not in listOfOpenStudies:
            nameAlreadyInUse=0
        else:
            i = i+1

    theStudy = myStudyManager.NewStudy(aStudyName)
    theStudyId = theStudy._get_StudyId()
    print aStudyName, theStudyId
    return theStudyId

    #--------------------------------------------------------------------------

salome_study_initial = 1

def salome_study_init(theStudyId=0):
    """
    Performs only once study creation or connection.
    optional argument : theStudyId
      When in embedded interpreter inside IAPP, theStudyId is not used
      When used without GUI (external interpreter)
        0      : create a new study (default).
        n (>0) : try connection to study with Id = n, or create a new one
                 if study not found.
    """

    global salome_study_initial
    global myStudyManager, myStudyId, myStudy, myStudyName
    global orb, lcc, naming_service, cm

    if salome_study_initial:
        salome_study_initial = 0

        orb, lcc, naming_service, cm = salome_kernel.salome_kernel_init()

        # get Study Manager reference
        if verbose(): print "looking for studyManager ..."
        obj = naming_service.Resolve('myStudyManager')
        myStudyManager = obj._narrow(SALOMEDS.StudyManager)
        if verbose(): print "studyManager found"

        # get active study Id, ref and name
        myStudyId = getActiveStudy(theStudyId)
        if verbose(): print "myStudyId",myStudyId
        myStudy = myStudyManager.GetStudyByID(myStudyId)
        myStudyName = myStudy._get_Name()

    return myStudyManager, myStudyId, myStudy, myStudyName




#! /usr/bin/env python
#  -*- coding: iso-8859-1 -*-
# Copyright (C) 2007-2012  CEA/DEN, EDF R&D, OPEN CASCADE
#
# Copyright (C) 2003-2007  OPEN CASCADE, EADS/CCR, LIP6, CEA/DEN,
# CEDRAT, EDF R&D, LEG, PRINCIPIA R&D, BUREAU VERITAS
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
#

#  SALOME Utils : general SALOME's definitions and tools
#  File   : Utils_Identity.py
#  Author : Estelle Deville, CEA
#  Module : SALOME
#  $Header: /home/server/cvs/KERNEL/KERNEL_SRC/src/Utils/Utils_Identity.py,v 1.8.2.4.14.3.12.1 2012-04-12 14:05:32 vsr Exp $
## @package Utils_Identity
# \brief Module to get information about user and version
#
import sys
import os
import socket

if not sys.platform == "win32":
    import pwd

import time
import string

def getShortHostName():
    """
    gives Hostname without domain extension.

    SALOME naming service needs short Hostnames (without domain extension).
    HOSTNAME is not allways defined in environment,
    socket.gethostname() gives short or complete Hostname, depending on
    defined aliases.
    """
    return string.split(socket.gethostname(),'.')[0]

class Identity:
    def __init__(self,name):
        self._name = name
        self._pid =  os.getpid()
        self._machine = socket.gethostname()
        self._adip =  socket.gethostbyname(self._machine) # IP adress        
        if sys.platform == "win32":
          self._uid  = os.getpid() 
          self._pwname = os.environ["USERNAME"]
        else:
          self._uid = os.getuid()
          list = pwd.getpwuid(self._uid)
          self._pwname  = list[0] # user name

        self._tc_start = time.time()
        self._cstart    = time.ctime(self._tc_start)
        self._cdir = os.getcwd()

def getapplipath():
    """
      Gives short application path (the complete path is $HOME/$APPLI)
    """
    return os.environ.get("APPLI",".salome_"+versnb)

try:
  file = open(os.path.join(os.environ["KERNEL_ROOT_DIR"],"bin","salome","VERSION"), "r")
  s = file.readline()
  versnb = string.strip(string.split(s, ":")[1])
  dirname=".salome_"+versnb
except:
  versnb = ""
  dirname=".salome"

def version():
    """
      Gives salome version number
    """
    return versnb



# -*- coding: utf-8 -*-
#
# Copyright (C) 2007-2012  CEA/DEN, EDF R&D, OPEN CASCADE
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
#

#=============================================================================
#  Author    : Guillaume Boulant (CSSI)
#  Rewritten by Renaud Barate (EDF R&D)
#  Project   : SALOME
#  Copyright : EDF 2001-2009
#  $Header: /home/server/cvs/KERNEL/KERNEL_SRC/src/KERNEL_PY/kernel/Attic/logger.py,v 1.1.8.3.12.1 2012-04-12 14:05:14 vsr Exp $
#=============================================================================
"""
This module defines a class which provides logging facility in Salome:
"""

import sys, os
import logging

from salome.kernel.deprecation import deprecated
from salome.kernel import termcolor
import salome.kernel.logconfig

class Logger(logging.Logger):
    """
    This class formats and displays log messages in Salome environment. It
    inherits :class:`Logger<logging.Logger>` class defined in :mod:`logging`
    module from Python library, so all methods from :class:`logging.Logger`
    can be used here. The format of the traces is:
    
    LEVEL    [keyword] : Message
    
    where `LEVEL` is the level of the message (`DEBUG`, `INFO`, etc.),
    `keyword` is the name of the logger, and `Message` is the message to log.
    
    When creating a new Logger object, the parameter `keyword` defines the
    name of the logger, `level` defines the logging level (default is
    :const:`logging.DEBUG` if KERNEL module is configured with --enable-debug
    option or :const:`logging.WARNING` otherwise), and `color` defines the color
    of the log messages for this logger (log messages will appear in color
    only when displayed on color-capable ASCII terminals). See module
    :mod:`salome.kernel.termcolor` for the color constants.
    
    By default, log messages will be displayed only on standard output. They
    can also be recorded in a file (see method :meth:`setLogFile`). For now,
    the CORBA-based logging facility can not be used through this class.

    A source filename `sourceFileName` can be defined. If this argument is
    specified, then the `keyword` is modified to the basename of the `sourceFileName`
    
    Basic usage::
    
        from salome.kernel.logger import Logger
        log = Logger("Test")
        log.debug("Debug message")
        log.info("Information message")
        log.warning("Warning message")
        log.error("Error message")
        log.critical("Fatal error message")

    """

    def __init__(self, keyword = "KEY", level = salome.kernel.logconfig.loggingLevel,
                 color = None, sourceFileName=None):

        if sourceFileName is not None:
            keyword = os.path.basename(sourceFileName).split('.')[0]
        logging.Logger.__init__(self, keyword, level)
        self._baseFormatString = "%(levelname)-8s [%(name)s] : %(message)s"
        self._baseFormatter = logging.Formatter(self._baseFormatString)
        if hasattr(sys.stdout, "flush"):
            self._stdoutStream = sys.stdout
        else:
            self._stdoutStream = _UnFlushableLogStream(sys.stdout)
        self._stdoutHandler = logging.StreamHandler(self._stdoutStream)
        self._stdoutHandler.setLevel(logging.DEBUG)
        self.setColor(color)
        self.addHandler(self._stdoutHandler)
        self._fileHandler = None

    def showDebug(self):
        """
        Log all messages, including DEBUG level messages (equivalent to
        ``setLevel(logging.DEBUG)``).
        """
        self.setLevel(logging.DEBUG)

    def setLogFile(self, logFilename):
        """
        Define a log file to record the log messages (in addition to the
        standard output).
        """
        self.closeLogFile()
        self._fileHandler = logging.FileHandler(logFilename, 'w')
        self._fileHandler.setLevel(logging.DEBUG)
        self._fileHandler.setFormatter(self._baseFormatter)
        self.addHandler(self._fileHandler)

    def setColor(self, color):
        """
        Set the color of log messages on color-capable terminals. If `color`
        is :const:`None`, the default color will be used.
        """
        if color is None or not termcolor.canDisplayColor(self._stdoutStream):
            stdoutFormatter = self._baseFormatter
        else:
            format = ("%s%s%s" %
                      (termcolor.getControlSequence(color),
                       self._baseFormatString,
                       termcolor.getControlSequence(termcolor.DEFAULT)))
            stdoutFormatter = logging.Formatter(format)
        self._stdoutHandler.setFormatter(stdoutFormatter)

    def closeLogFile(self):
        """Close the log file."""
        if self._fileHandler is not None:
            self.removeHandler(self._fileHandler)
            self._fileHandler.close()
            self._fileHandler = None

    def hideDebug(self):
        """
        Hide DEBUG level messages (equivalent to ``setLevel(logging.INFO)``).
        """
        self.setLevel(logging.INFO)

    @deprecated("Deprecated since version 5.1.5. Please replace with "
                "Logger.critical(message)")
    def fatal(self, message):
        """
        Log a message with CRITICAL level. This method only exists for
        backward compatibility and is equivalent to ``critical(message)``.
        """
        self.critical(message)


class _UnFlushableLogStream:
    """
    This utility class allows to log messages to a stream with no `flush`
    method. This is useful to send log messages to `PyOut` objects.
    """

    def __init__(self, stream):
        self._stream = stream

    def write(self, msg):
        self._stream.write(msg)

    def flush(self):
        pass


class ExtLogger(Logger):
    """
    This class extends :class:`Logger` class and adds exception information
    when DEBUG messages are recorded. It exists mainly for backward
    compatibility, as the same thing can be done by calling
    ``Logger.debug(message, exc_info = True)``.
    """
    
    @deprecated("Class ExtLogger is deprecated since version 5.1.5. See "
                "documentation for replacement.")
    def __init__(self, keyword = "KEY",
                 level = salome.kernel.logconfig.loggingLevel,
                 color = None, sourceFileName=None):
        Logger.__init__(self, keyword, level, color, sourceFileName)

    def debug( self, message ):
        """
        Log a DEBUG message with exception information (equivalent to
        ``Logger.debug(message, exc_info = True)``).
        """
        Logger.debug(self, message, exc_info = True)


def TEST_Logger():
    """Test function for logger module"""
    log = Logger("TST")

    # Base methods
    log.info("Information message")
    log.debug("Debug message")
    log.fatal("Fatal error message")

    # Message building
    data = 12
    log.info("This message displays data = " + str(data))

    data = {}
    data["KERNEL"] = "V1"
    data["GEOM"] = "V2"
    log.info("This message displays data = " + str(data))

    # Test with a non-string parameter
    log.info(data)

    # Test with a default instance
    log = Logger()
    log.info("Default logger")

    # Test showDebug method
    log.setLogFile("test.log")
    log.debug("Debug trace")
    log.hideDebug()
    log.debug("This trace should NOT be displayed")
    log.showDebug()
    log.debug("This trace should be displayed")
    log.closeLogFile()
    log.info("After closing the log file")


# Main function only used to test the module
if __name__ == "__main__":
    TEST_Logger()



# -*- coding: iso-8859-1 -*-
# Copyright (C) 2007-2012  CEA/DEN, EDF R&D, OPEN CASCADE
#
# Copyright (C) 2003-2007  OPEN CASCADE, EADS/CCR, LIP6, CEA/DEN,
# CEDRAT, EDF R&D, LEG, PRINCIPIA R&D, BUREAU VERITAS
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# See http://www.salome-platform.org/ or email : webmaster.salome@opencascade.com
#

__author__="gboulant"
__date__ ="$21 mai 2010 18:00:23$"


def findFiles(rootpath, excludes=[]):
    """
    This looks after files recursively from the specified rootpath,
    but without visiting directories whose basename is in the list
    @param excludes.
    """
    if not os.path.exists(rootpath):
        raise RuntimeError("the path %s does not exists"%rootpath)

    exclude_options=""
    for excludepath in excludes:
        exclude_options+="-e %s "%excludepath

    listfiles=[]        
    stream=os.popen("find %s -type f | grep -v -e '\.svn' %s 2>/dev/null"%(rootpath,exclude_options))
    for line in stream.readlines():
        listfiles.append(line.split('\n')[0])
        
    return listfiles

import sys
from stat import ST_MODE, S_ISDIR, S_ISREG
def walktree(rootpath, callback, **kwargs):
    '''
    This recursively descends the directory tree rooted at rootpath,
    calling the callback function for each regular file
    '''
    for f in os.listdir(rootpath):
        pathname = os.path.join(rootpath, f)
        try:
            mode = os.stat(pathname)[ST_MODE]
        except OSError, e:
            # It probably means that the file is a broken inode
            mode = -1
        if S_ISDIR(mode):
            # It's a directory, recurse into it
            walktree(pathname, callback, **kwargs)
        elif S_ISREG(mode):
            # It's a file, call the callback function
            callback(pathname, **kwargs)
        else:
            # Unknown file type, print a message
            print 'Skipping %s' % pathname


#
# =============================================================
# Use cases and unit test functions
# =============================================================
#
import os
try:
    TESTDOCDIR=os.path.join(os.environ["KERNEL_ROOT_DIR"],"share")
except KeyError:
    TESTDOCDIR="/tmp"
    
def TEST_findFiles():
    print "########## find 1"
    rootpath=TESTDOCDIR
    listfiles=findFiles(rootpath)
    for filename in listfiles:
        print filename

    print "########## find 2"
    excludes=[os.path.join(TESTDOCDIR,"doc")]
    listfiles=findFiles(rootpath,excludes)
    for filename in listfiles:
        print filename

    return True

# This is the test callback function
def visitfile_withargs(file, rootid):
    print 'visiting file %s (rootid=%s)'%(file,str(rootid))

def visitfile_withoutargs(file):
    print 'visiting file %s'%(file)

def TEST_walktree():
    #walktree(TESTDOCDIR, visitfile_withargs, rootid=2)
    walktree(TESTDOCDIR, visitfile_withoutargs)
    return True

if __name__ == "__main__":
    import unittester
    unittester.run("syshelper", "TEST_findFiles")
    unittester.run("syshelper", "TEST_walktree")
